<?php

namespace Teknisa\EST\Factories;

use Doctrine\ORM\EntityManager;

class InventarioRotativoFactory {
    protected $connection;

    public function __construct(EntityManager $entityManager) {
        $this->connection = $entityManager->getConnection();
    }

    public function getInventarioComLote($params) {
        $sql = "SELECT IDINVLOTEOK,
            NRORG
            FROM INVEROTALOTE
            WHERE(CDFILIAL = :P_CDFILIAL)
            AND(DTINVELOTE =
            (SELECT MAX(DTINVELOTE)
            FROM INVEROTALOTE
            WHERE (CDFILIAL = :P_CDFILIAL)
            AND (DTINVELOTE < :P_DTINVEROTA)
            ))
            AND (IDINVLOTEOK = 'N')";

        $std = $this->connection->prepare($sql);
        $std->execute($params);

        return $dataSet = $std->fetchAll();
    }

    public function getInventarioSemLote($params) {
        $sql = "SELECT IDINVENTOK,
            NRORG
            FROM INVEROTA
            WHERE(CDFILIAL = :P_CDFILIAL)
            AND(DTINVEROTA =
            (SELECT MAX(DTINVEROTA)
            FROM INVEROTA
            WHERE (CDFILIAL = :P_CDFILIAL)
            AND (DTINVEROTA < :P_DTINVEROTA)
            ))
            AND (IDINVENTOK = 'N')";

        $std = $this->connection->prepare($sql);
        $std->execute($params);

        return $dataSet = $std->fetchAll();
    }
    
    public function getParamFilial($cdfilial, $nrorg) {
        $sql = "SELECT * FROM PARAMFILIAL 
                WHERE CDFILIAL = :CDFILIAL 
                AND NRORG = :NRORG";

         $params = array(
                'CDFILIAL'=>$cdfilial,
                'NRORG'=>$nrorg
            );
            $std = $this->connection->prepare($sql);
            $std->execute($params);
            return $std->fetch();
    }
    
    public function getParamGeral($nrorg) {
        $sql = "SELECT * FROM PARAMGERAL 
                WHERE NRORG = :NRORG";

         $params = array(
                'NRORG'=>$nrorg
            );
            $std = $this->connection->prepare($sql);
            $std->execute($params);
            return $std->fetch();
    }
    
    public function getValidaInventario($params) {
        $sql = "SELECT I.CDFILIAL, I.DTCONTAGEM, I.CDALMOXARIFE, I.CDLOCALESTOQ, I.IDSITESTCEGO, I.IDSITUACEITEINV
                  FROM INVENTESTCEGO I,
               (SELECT CDFILIAL, MAX(DTCONTAGEM) AS DTCONTAGEM
                  FROM INVENTESTCEGO
                 WHERE DTCONTAGEM < :DTPROCESSA
                 GROUP BY CDFILIAL) X
                 WHERE I.CDFILIAL = :CDFILIAL
                   AND I.CDFILIAL = X.CDFILIAL
                   AND I.DTCONTAGEM = X.DTCONTAGEM
                   AND I.IDSITESTCEGO = 'AP'
                   AND I.IDSITUACEITEINV = 'P'
                 ORDER BY I.DTCONTAGEM";

        $std = $this->connection->prepare($sql);
        $std->execute($params);

        return $dataSet = $std->fetchAll();
    }
    
    public function getInventCegoInfo($params) {
        $sql = "SELECT DTCONTAGEM,
                       CDALMOXARIFE,
                       CDLOCALESTOQ,
                       IDSITESTCEGO,
                       IDSITUACEITEINV,
                       DTAPROVINVESTCE
                  FROM INVENTESTCEGO
                 WHERE CDFILIAL = :CDFILIAL
                   AND DTCONTAGEM = :DTCONTAGEM
                   AND IDSITESTCEGO IN ( 'PR', 'ED', 'DG' )
                   AND SYSDATE >= :DTCONTAGEM";

        $std = $this->connection->prepare($sql);
        $std->execute($params);

        return $dataSet = $std->fetchAll();
    }
    
    public function getGroupOperatorInfo($params) {
        $sql = "SELECT IDAUTGRUPOP
                  FROM AUTGRUPOP  A,
                       GRUPOPER   G,
                       OPERGRUPOP O
                 WHERE A.CDGRUPOPER = G.CDGRUPOPER
                   AND G.CDGRUPOPER = O.CDGRUPOPER
                   AND O.CDOPERADOR = :CDOPERADOR
                   AND IDAUTGRUPOP IN ( 'GEST', 'GGER', 'GAUD' )";

        $std = $this->connection->prepare($sql);
        $std->execute($params);

        return $dataSet = $std->fetchAll();
    }
    
}