<?php
namespace Fiscal\Logic\Impostos;

use \Fiscal\Entities\Itemproimpos;
use \Fiscal\Entities\Notafiscal;
use \Fiscal\Entities\Trabitproimpos;
use \Geral\Entities\Tipooperac;
use \Indeterminado\Entities\Impobrigtpoper;
use \Setforms2\Runtime\Helper\Procedure;
use \Setforms2\Runtime\Helper\Procedure\Param;

/**
 * Executa as procedures de geracao de impostos da nota fiscal.
 *
 * Created by JetBrains PhpStorm.
 * User: paulopereira
 * Date: 24/10/12
 * Time: 12:59
 * To change this template use File | Settings | File Templates.
 */
class GeradorImposto{

    /**
     * Define os processos de impostos de entrda.
     */
    const IMPOSTO_ENTRADA = "IMPOSTO_ENTRADA";
    /**
     * Define os processos de impostos de saida.
     */
    const IMPOSTO_SAIDA = "IMPOSTO_SAIDA";
    /**
     * Operação de compra.
     */
    const OPERACAO_COMPRA = '01';
    /**
     * Operação de tranferência.
     */
    const OPERACAO_TRANSF = '02';
    /**@var $procedure Procedure */
    private static $procedure;
    private static $retorno;

    /**
     * Executa as procedures de IMPOSTO DE ENTRADA ou IMPOSTO DE SAIDA, dependendo de qual for solicitado pelo cliente
     * da classe. Os tipos diferentes podem ser identificados pelas constrantes IMPOSTO_ENTRADA e IMPOSTO_SAIDA.
     * Atentar-se para o fato de que o parametro $organizacao se refere ao codigo do fornecedor, no caso da
     * IMPOSTO_ENTRADA e cliente, no caso da IMPOSTO_SAIDA.
     *
     * @static
     *
     * @param Notafiscal $notaFiscal
     * @param string     $tpImposto
     *
     * @throws \Exception
     * @return float Valor da Nota Fiscal
     */
    public static function callProcedureImposto(Notafiscal $notaFiscal, $tpImposto = self::IMPOSTO_ENTRADA){
        $empresa  = $notaFiscal->getEmpresa()->getCdempresa();
        $inscesta = $notaFiscal->getInscesta()->getCdinscesta();
        $tipooper = $notaFiscal->getEntrsaidop()->getCdtipooper();
        /*if($tpImposto === self::IMPOSTO_ENTRADA){
            $organizacao = $tipooper == self::OPERACAO_COMPRA ? $notaFiscal->getForneced()->getCdforneced()
                                                              : $notaFiscal->getCdfiliemit();
        } else {
            $organizacao = $tipooper == self::OPERACAO_COMPRA ? $notaFiscal->getCliente()->getCdcliente()
                                                              : $notaFiscal->getCdfilirece();
        }*/

        switch ($notaFiscal->getIdorigdestnf()){
            case Notafiscal::ID_ORIGEM_DESTINO_FORNECEDOR :
                $organizacao = $notaFiscal->getForneced()->getCdforneced();
                break;
            case Notafiscal::ID_ORIGEM_DESTINO_FILIAL :
                $organizacao = $notaFiscal->getCdfiliemit();
                break;
            case Notafiscal::ID_ORIGEM_DESTINO_CLIENTE :
                $organizacao = $notaFiscal->getCdcliente();
                break;
        }

        $filial     = $notaFiscal->getFilial()->getCdfilial();
        $lanctonf   = $notaFiscal->getNrlanctonf();
        $inivalid   = $notaFiscal->getDtentrsaid();
        $entrsaid   = $notaFiscal->getEntrsaidop()->getIdentrsaidop();
        $cdoperador = \Geral\Util\Ambiente::getCdOperador();
        $audit      = 'N';

        $tipoOperac = \Geral\Entities\Tipooperac::findOneByCdtipooperAndIdentrsaidopAndNrorg($tipooper, $entrsaid,
            \Geral\Util\Ambiente::getNrorg());

        if ($tipoOperac->getIdatuesttpop() == 'S'){
            /* @var $lancamentoEstoque \Estoque\Entities\Lanctoestoq */
            $lancamentoEstoque = \Estoque\Entities\Lanctoestoq::findOneByCdfilialAndNrlanctonfAndNrorg($filial,
                $lanctonf, $notaFiscal->getNrorg());

            if ($lancamentoEstoque instanceof \Estoque\Entities\Lanctoestoq){
                $lancestq = $lancamentoEstoque->getNrlancestq();
            }
            else{
                $lancestq = \Geral\Util\NovoCodigo::geraCodigo(
                    \Geral\Util\Ambiente::getNrorg(),
                    \Estoque\Entities\Lanctoestoq::CDCONTADOR_LANCTOESTOQ.\Geral\Util\Ambiente::getCdFilial());
            }
        }
        else
            $lancestq = '';

        try{
            self::$procedure = new Procedure($tpImposto);
            self::$procedure->addParam(
                new Param (
                    "P_EMPRESA",
                    Param::PARAM_INPUT,
                    $empresa,
                    Param::PARAM_TYPE_STR
                )
            );
            self::$procedure->addParam(
                new Param(
                    "P_INSCESTA",
                    Param::PARAM_INPUT,
                    $inscesta,
                    Param::PARAM_TYPE_STR
                )
            );
            self::$procedure->addParam(
                new Param(
                    "P_TIPOOPER",
                    Param::PARAM_INPUT,
                    $tipooper,
                    Param::PARAM_TYPE_STR
                )
            );
            self::$procedure->addParam(
                new Param(
                    "P_FORNECED",
                    Param::PARAM_INPUT,
                    $organizacao,
                    Param::PARAM_TYPE_STR
                )
            );
            self::$procedure->addParam(
                new Param(
                    "P_FILIAL",
                    Param::PARAM_INPUT,
                    $filial,
                    Param::PARAM_TYPE_STR
                )
            );
            self::$procedure->addParam(
                new Param(
                    "P_LANCTONF",
                    Param::PARAM_INPUT,
                    $lanctonf,
                    Param::PARAM_TYPE_STR
                )
            );
            self::$procedure->addParam(
                new Param(
                    "P_INIVALID",
                    Param::PARAM_INPUT,
                    /** @var $inivalid \DateTime */
                    $inivalid->format(\Geral\Util\DateUtil::FORMATO_BRASILEIRO_DATAHORA),
                    Param::PARAM_TYPE_STR
                )
            );
            self::$procedure->addParam(
                new Param(
                    "P_VRNOTAFISC",
                    Param::PARAM_OUTPUT,
                    self::$retorno,
                    Param::PARAM_TYPE_INT,
                    20
                )
            );
            self::$procedure->addParam(
                new Param(
                    "P_LANCESTQ",
                    Param::PARAM_INPUT,
                    $lancestq,
                    Param::PARAM_TYPE_STR
                )
            );
            self::$procedure->addParam(
                new Param(
                    "P_ENTRSAID",
                    Param::PARAM_INPUT,
                    $entrsaid,
                    Param::PARAM_TYPE_STR
                )
            );
            self::$procedure->addParam(
                new Param(
                    "P_CDOPERADOR",
                    Param::PARAM_INPUT,
                    $cdoperador,
                    Param::PARAM_TYPE_STR
                )
            );
            self::$procedure->addParam(
                new Param(
                    "P_AUDIT",
                    Param::PARAM_INPUT,
                    $audit,
                    Param::PARAM_TYPE_STR
                )
            );

            return self::$procedure->call();
        } catch (\Exception $e){
            throw $e;
        }
        //        try {
        //            self::$procedure = new Procedure($tpImposto);
        //
        //            $empresa     = $notaFiscal->getEmpresa()->getCdempresa();
        //            $inscesta    = $notaFiscal->getInscesta()->getCdinscesta();
        //            $tipooper    = $notaFiscal->getEntrsaidop()->getCdtipooper();
        //            $organizacao = $tpImposto === self::IMPOSTO_ENTRADA ? $notaFiscal->getForneced()->getCdforneced()
        //                                                                : $notaFiscal->getCliente()->getCdcliente();
        //            $filial      = $notaFiscal->getFilial()->getCdfilial();
        //            $lanctonf    = $notaFiscal->getNrlanctonf();
        //            $inivalid    = $notaFiscal->getDtentrsaid();
        //            $dtentrsaid  = $notaFiscal->getDtentrsaid();
        //            $cdoperador  = \Geral\Util\Ambiente::getCdOperador();
        //            $audit       = $cdoperador;
        //            $nrorg       = \Geral\Util\Ambiente::getNrorg();
        //            $lancestq    = \Estoque\Entities\Lanctoestoq::findOneBycdfilialAndnrlanctonfAndNrorg (
        //                $notaFiscal->getFilial()->getCdfilial(), $notaFiscal->getNrlanctonf(), $nrorg
        //            );
        //
        //            if ($tpImposto === self::IMPOSTO_SAIDA) {
        //                self::adicionaParametrosImpostoSaida(
        //                    $empresa, $inscesta, $tipooper, $organizacao, $filial, $lanctonf, $inivalid, $lancestq, $cdoperador
        //                );
        //            } else {
        //                self::adicionaParametrosImpostoEntrada (
        //                    $empresa, $inscesta, $tipooper, $organizacao, $filial, $lanctonf, $inivalid, $lancestq, $dtentrsaid,
        //                    $cdoperador, $audit
        //                );
        //            }
        //
        //            self::$procedure->call();
        ////            return self::$retorno;
        //        } catch (\Exception $e) {
        //            throw $e;
        //        }
    }

    /**
     * Executa a procedure de GRAVA_IMPOSTO_TEMPORARIO.
     *
     * @static
     *
     * @param \Fiscal\Entities\Notafiscal $notaFiscal
     */
    public static function callProcedureGravaImpostoTemporario(Notafiscal $notaFiscal){
        $cdfilial   = $notaFiscal->getFilial()->getCdfilial();
        $nrlanctonf = $notaFiscal->getNrlanctonf();

        self::$procedure = new Procedure('GRAVA_IMPOSTO_TEMPORARIO');
        self::adicionaParametrosGravaImpostoTemporario($cdfilial, $nrlanctonf);
        self::$procedure->call();
    }

    /**
     * Parametros usados pela procedure GRAVA_IMPOSTO_TEMPORARIO
     *
     * @static
     *
     * @param $cdfilial
     * @param $nrlanctonf
     */
    protected static function adicionaParametrosGravaImpostoTemporario($cdfilial, $nrlanctonf){
        self::$procedure->addParam(
            new Param(
                'P_FILIAL',
                Param::PARAM_INPUT,
                $cdfilial,
                Param::PARAM_TYPE_STR
            )
        );
        self::$procedure->addParam(
            new Param(
                'P_LANCTONF',
                Param::PARAM_INPUT,
                $nrlanctonf,
                Param::PARAM_TYPE_STR
            )
        );
    }

    /**
     * Executar procedure de Gravar imposto.
     *
     * @static
     *
     * @param \Fiscal\Entities\Notafiscal $notafiscal
     *
     * @throws \Exception
     */
    public static function callProcedureGravaImposto(Notafiscal $notafiscal){
        $cdfilial   = $notafiscal->getFilial()->getCdfilial();
        $nrlanctonf = $notafiscal->getNrlanctonf();

        try{
            self::$procedure = new Procedure("GRAVA_IMPOSTO");
            self::adicionaParametrosGravaImposto($cdfilial, $nrlanctonf);
            self::$procedure->call();
        } catch (\Exception $e){
            throw $e;
        }
    }

    /**
     * Parametros usados pela procedure GRAVA_IMPOSTO.
     *
     * @static
     *
     * @param $cdfilial
     * @param $nrlanctonf
     */
    protected static function adicionaParametrosGravaImposto($cdfilial, $nrlanctonf){
        self::$procedure->addParam(
            new Param (
                'P_FILIAL',
                Param::PARAM_INPUT,
                $cdfilial,
                Param::PARAM_TYPE_STR
            )
        );
        self::$procedure->addParam(
            new Param(
                'P_LANCTONF',
                Param::PARAM_INPUT,
                $nrlanctonf,
                Param::PARAM_TYPE_STR
            )
        );
    }

    /**
     * Passa os valores para os parametros da procedure.
     *
     * @static
     *
     * @param string $empresa
     * @param string $inscesta
     * @param string $tipooper
     * @param string $cliente
     * @param string $filial
     * @param string $lanctonf
     * @param string $inivalid
     * @param string $lancestq
     * @param string $cdoperador
     */
    protected static function adicionaParametrosImpostoSaida($empresa, $inscesta, $tipooper, $cliente, $filial,
                                                             $lanctonf, $inivalid, $lancestq, $cdoperador){
        self::adicionaParametrosPadrao(
            $empresa, $inscesta, $tipooper, $filial, $lanctonf, $inivalid, $lancestq, $cdoperador
        );
        self::$procedure->addParam(
            new Param(
                "P_CLIENTE",
                Param::PARAM_INPUT,
                $cliente,
                Param::PARAM_TYPE_STR
            )
        );
    }

    /**
     * Passa os valores para os parametros da procedure. Como a procedure de IMPOSTO_ENTRADA recebe todos os parametros
     * de IMPOSTO_SAIDA, este metodo chama o metodo de adicionaParametrosImpostoSaida, para aproveitar sua atribuicao de
     * valores.
     *
     * @static
     *
     * @param $empresa
     * @param $inscesta
     * @param $tipooper
     * @param $fornecedor
     * @param $filial
     * @param $lanctonf
     * @param $inivalid
     * @param $lancestq
     * @param $entrsaid
     * @param $cdoperador
     * @param $audit
     */
    protected static function adicionaParametrosImpostoEntrada($empresa, $inscesta, $tipooper, $fornecedor, $filial,
                                                               $lanctonf, $inivalid, $lancestq, $entrsaid,
                                                               $cdoperador, $audit){
        self::adicionaParametrosPadrao(
            $empresa, $inscesta, $tipooper, $filial, $lanctonf, $inivalid, $lancestq, $cdoperador
        );
        self::$procedure->addParam(
            new Param(
                "P_ENTRSAID",
                Param::PARAM_INPUT,
                $entrsaid,
                Param::PARAM_TYPE_STR
            )
        );
        self::$procedure->addParam(
            new Param(
                "P_AUDIT",
                Param::PARAM_INPUT,
                $audit,
                Param::PARAM_TYPE_STR
            )
        );
        self::$procedure->addParam(
            new Param(
                "P_FORNECED",
                Param::PARAM_INPUT,
                $fornecedor,
                Param::PARAM_TYPE_STR
            )
        );
    }

    /**
     * Parametros comuns para as procedures de imposto de entrada e saida.
     *
     * @static
     *
     * @param string $empresa
     * @param string $inscesta
     * @param string $tipooper
     * @param string $filial
     * @param string $lanctonf
     * @param string $inivalid
     * @param string $lancestq
     * @param string $cdoperador
     */
    protected static function adicionaParametrosPadrao($empresa, $inscesta, $tipooper, $filial, $lanctonf, $inivalid,
                                                       $lancestq, $cdoperador){
        self::$procedure->addParam(
            new Param(
                "P_EMPRESA",
                Param::PARAM_INPUT,
                $empresa,
                Param::PARAM_TYPE_STR
            )
        );
        self::$procedure->addParam(
            new Param(
                "P_INSCESTA",
                Param::PARAM_INPUT,
                $inscesta,
                Param::PARAM_TYPE_STR
            )
        );
        self::$procedure->addParam(
            new Param(
                "P_TIPOOPER",
                Param::PARAM_INPUT,
                $tipooper,
                Param::PARAM_TYPE_STR
            )
        );
        self::$procedure->addParam(
            new Param(
                "P_FILIAL",
                Param::PARAM_INPUT,
                $filial,
                Param::PARAM_TYPE_STR
            )
        );
        self::$procedure->addParam(
            new Param(
                "P_LANCTONF",
                Param::PARAM_INPUT,
                $lanctonf,
                Param::PARAM_TYPE_STR
            )
        );
        self::$procedure->addParam(
            new Param (
                "P_INIVALID",
                Param::PARAM_INPUT,
                $inivalid,
                Param::PARAM_TYPE_STR
            )
        );

        self::$procedure->addParam(
            new Param(
                "P_VRNOTAFISC",
                Param::PARAM_OUTPUT,
                self::$retorno,
                Param::PARAM_TYPE_INT,
                5
            )
        );
        self::$procedure->addParam(
            new Param (
                "P_LANCESTQ",
                Param::PARAM_INPUT,
                $lancestq,
                Param::PARAM_TYPE_STR
            )
        );
        self::$procedure->addParam(
            new Param(
                "P_CDOPERADOR",
                Param::PARAM_INPUT,
                $cdoperador,
                Param::PARAM_TYPE_STR
            )
        );
    }
}